#include <stdint.h>
#include <stdlib.h>
#include <avr/sleep.h>

#define F_CPU 8000000ul
#include <util/delay.h>

#include "hal/led.h"
#include "hal/battery.h"


#define CANDLE_MAX 65    
#define CANDLE_DOWN 15
#define CANDLE_UP 5
#define CANDLE_SPEED 10
#define CANDLE_LOWPASS_DOWN 3
#define CANDLE_LOWPASS_UP 5


/* Domain of color index */
/* Constant color domain: 512 */
/* Two color domain: 1024 */
/* Full domain: 1536 */
#define STAR_DOMAIN 1024

/* color index start */
/* Red: 256 */
/* Green: 1280 */
/* Blue: 768 */
#define STAR_START 1280


/* Frequency of new generated colors [1 - 1000] */
#define STAR_FREQUENCY 20
#define STAR_LOWPASS 1



/* LED values [0, 127] */
volatile uint8_t led[13];
volatile uint8_t battery_low = 0;
volatile uint8_t battery_running = 0;


uint8_t color(uint16_t frame){
	
	
	if(frame < 256){
		
		return frame;
	}
	else if(frame <= 768){
		
		return 255;
	}
	else if(frame < 1024){
		
		return 1024 - frame;
	}
	else{
		
		return 0;
	}
}

void generate(uint8_t * data){
	
	uint16_t frame;
	
	frame = ((rand() % STAR_DOMAIN) + STAR_START) % STAR_DOMAIN;
			
	data[0] = color(frame) / 6;
	data[1] = color((frame +  512) % 1536) / 6;
	data[2] = color((frame +  1024) % 1536) / 6;
}


int main(){
	
	int8_t i;
	
	int8_t led_nominal[13];
	
	for(i = 0; i < 13; ++i){
	
		led[i] = 0;
		led_nominal[i] = 0;
	}
	
	generate((uint8_t *) led_nominal);
	
	
	led_init();
	led_enable();
	battery_init();
	
	
	while(1){
	
		battery_start();
		
		while(battery_running){
		}
		
		if(battery_low >= 10){
			
			for(i = 0; i < 13; ++i){
			
					led[i] = 0;
			}
			
			for(i = 0; i < 10; ++i){
				
				led[0] = 127;
				_delay_ms(500);
				led[0] = 0;
				_delay_ms(500);
			}
			
			set_sleep_mode(SLEEP_MODE_PWR_DOWN);
			sleep_mode();
		}
		else{
			
			/* CANDLE */
			
			for(i = 3; i < 13; ++i){
		
				if (led_nominal[i] > rand() % CANDLE_MAX){
					led_nominal[i] -= rand() % CANDLE_DOWN;
				}
				else{
					led_nominal[i] += rand() % CANDLE_UP;			
				}
		
				if(led_nominal[i] < 0){
					led_nominal[i] = 0;
				}
			
				if(led_nominal[i] - led[i] < 0){
					led[i] += (led_nominal[i] - led[i]) / CANDLE_LOWPASS_DOWN;
				}
				else{
					led[i] += (led_nominal[i] - led[i]) / CANDLE_LOWPASS_UP;
				}
			}
			
			/* STAR */
			
			if((rand() % 1000) < STAR_FREQUENCY){
			
				generate((uint8_t *) led_nominal);
			}
			
			for(i = 0; i < 3; ++i){
				
				if(led[i] > led_nominal[i]){
					
					led[i] -= STAR_LOWPASS;
				}
				else{
					
					if(led[i] < led_nominal[i]){
					
						led[i] += STAR_LOWPASS;
					}
				}
			}
			
		
			_delay_ms(CANDLE_SPEED);
		}
	}
	
	return 0;
}


